﻿namespace MyCompany.Commerce.Runtime.MyExtendedAuthService
{
    using Microsoft.Dynamics.Commerce.Runtime;
    using Microsoft.Dynamics.Commerce.Runtime.Handlers;
    using Microsoft.Dynamics.Commerce.Runtime.Messages;
    using Microsoft.Dynamics.Commerce.Runtime.Services.Messages;
    using System;
    using System.Collections.Generic;
    using System.Globalization;

    public class UniqueSecretExtendedAuthenticationService : INamedRequestHandler
    {
        public IEnumerable<Type> SupportedRequestTypes
        {
            get
            {
                return new[]
                {
                        typeof(GetUserEnrollmentDetailsServiceRequest),
                        typeof(GetUserAuthenticationCredentialIdServiceRequest),
                        typeof(ConfirmUserAuthenticationServiceRequest)
                };
            }
        }

        public string HandlerName
        {
            get
            {
                return "auth://example.auth.contoso.com/msr";
            }
        }

        public Response Execute(Request request)
        {
            if (request == null)
            {
                throw new ArgumentNullException("request");
            }

            Response response;
            Type requestType = request.GetType();

            if (requestType == typeof(GetUserEnrollmentDetailsServiceRequest))
            {
                response = this.GetUserEnrollmentDetails((GetUserEnrollmentDetailsServiceRequest)request);
            }
            else if (requestType == typeof(GetUserAuthenticationCredentialIdServiceRequest))
            {
                response = this.GetUserAuthenticationCredentialId((GetUserAuthenticationCredentialIdServiceRequest)request);
            }
            else if (requestType == typeof(ConfirmUserAuthenticationServiceRequest))
            {
                response = this.ConfirmUserAuthentication((ConfirmUserAuthenticationServiceRequest)request);
            }
            else
            {
                throw new NotSupportedException(string.Format(CultureInfo.InvariantCulture, "Request '{0}' is not supported.", request));
            }

            return response;
        }

        private GetUserAuthenticationCredentialIdServiceResponse GetUserAuthenticationCredentialId(GetUserAuthenticationCredentialIdServiceRequest request)
        {
            return this.GetUserAuthenticationCredentialId(request.Credential, request.RequestContext);
        }

        private GetUserAuthenticationCredentialIdServiceResponse GetUserAuthenticationCredentialId(string credential, RequestContext requestContext)
        {
            // TODO: this is the place where you can customize the mapping between what was scanned/swiped and what the credential id is in the StaffCredentialsview
            string credentialId = credential;

            return new GetUserAuthenticationCredentialIdServiceResponse(credentialId);
        }

        private GetUserEnrollmentDetailsServiceResponse GetUserEnrollmentDetails(GetUserEnrollmentDetailsServiceRequest request)
        {
            string credentialId = this.GetUserAuthenticationCredentialId(request.Credential, request.RequestContext).CredentialId;
            return new GetUserEnrollmentDetailsServiceResponse(credentialId, string.Empty);
        }

        private Response ConfirmUserAuthentication(ConfirmUserAuthenticationServiceRequest request)
        {
            return new NullResponse();
        }
    }
}
